/********************************************************************************/
/*  Basecode for Discovery 32F4 board										  	*/
/*  Board: STM32F407 Discovery				             					  	*/
/*  Manufacture: ST		                                                   	  	*/
/*	COPYRIGHT (C) 2012		Ing. Thomas Winkler	<t.winkler@aon.at>				*/
/*  Built with CooCox CoIDE v1.5.0  											*/
/*  version 0.09.00																*/
/********************************************************************************/
/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License only.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   wrapper.c: wrapper from fatfs to stdio functions
*/

#include "stm32f4xx.h"

#include <stdio.h>
#include <stdarg.h>
#include <malloc.h>

#include "ff.h"
#include "fswrapper.h"






//
// STDIO wrapper ##########
//


static 	FATFS	FileSystemObject;
static	FIL		fhTab[MAX_HANDLES];


//
// fopen replacement for stdio compatibility
//
FIL *__fopen(const char *filename, const char *omode)
{
	/*Mode Description :: http://www.warpspeed.com.au/cgi-bin/inf2html.cmd?..\html\book\Toolkt40\XPG4REF.INF+126
	r Open a text file for reading. The file must exist.
	w Create a text file for writing. If the given file exists, its contents are destroyed.
	a Open a text file in append mode for writing at the end of the file. fopen creates the file if it does not exist.
	r+ Open a text file for both reading and writing. The file must exist.
	w+ Create a text file for both reading and writing. If the given file exists, its contents are destroyed.
	a+ Open a text file in append mode for reading or updating at the end of the file. fopen creates the file if it does not exist.
	rb Open a binary file for reading. The file must exist.
	wb Create an empty binary file for writing. If the file exists, its contents are destroyed.
	ab Open a binary file in append mode for writing at the end of the file. fopen creates the file if it does not exist.
	r+b or rb+ Open a binary file for both reading and writing. The file must exist.
	w+b or wb+ Create an empty binary file for both reading and writing. If the file exists, its contents will be destroyed.
	a+b or ab+ Open a binary file in append mode for writing at the end of the file. fopen creates the file if it does not exist.

	Warning: Use the w, w+, wb, w+b, and wb+ parameters with care; data in existing files of the same name will be lost. */
	int		i;
	FIL		*fh = NULL;

	for(i=0; i<MAX_HANDLES; i++)
	{
		if(fhTab[i].fs == NULL)
		{
			fh = &fhTab[i];
			break;
		}
	}
	if(fh != NULL)
	{
		int		fRead	= 0;
		int		fWrite	= 0;
		int		fPlus	= 0;
//		int		fBinary	= 0;
		int		fAppend	= 0;
		int		fBad	= 0;
		BYTE	mode	= 0;
		char	c;

		while((c = *(omode++)) != 0)
		{
			switch(c)
			{
			case 'r':								// read mode
				fRead = 1;
				break;
			case 'w':								// write mode
				fWrite = 1;
				break;
			case '+':								// extends read+write
				fPlus = 1;
				break;
			case 'a':								// append mode
				fAppend = 1;
				break;
			case 'b':								// binary :: not yet implemented
//				fBinary = 1;
				break;
			default:
				fBad = 1;
				break;
			}
		}
		if(fBad)
			;
		else if(fRead)
		{
			if(fWrite == 0 && fAppend == 0)
			{
				mode  =	FA_READ | FA_OPEN_EXISTING |
						(fPlus ? FA_WRITE : 0);
			}
		}
		else if(fWrite)
		{
			if(fAppend == 0)
			{
				mode  =	FA_WRITE | FA_CREATE_ALWAYS |
						(fPlus ? FA_READ : 0);
			}
		}
		else if(fAppend)
		{
			// not fully implemented!! normally write is always at end of file ...
			mode  =	FA_WRITE | FA_OPEN_ALWAYS |
					(fPlus ? FA_READ : 0);
		}

		if(mode == 0)
			fh = NULL;
		else if(f_open(fh, filename, mode) != FR_OK)
			fh = NULL;
		else
		{
			// open successful!
			if(fAppend)
			{
				// set filepointer to end of file
				f_lseek(fh, f_size(fh));
			}
		}
		if(fh == NULL)
			fhTab[i].fs = NULL;
	}
	return fh;
}


//
// fclose replacement for stdio compatibility
//
int __fclose(FIL *fh)
{
	int rc = 0;
	if(f_close(fh) != FR_OK)
		rc = EOF;

	fh->fs = NULL;
	return rc;
}


//
// fseek replacement for stdio compatibility
//
int __fseek(FIL *stream, long int offset, int origin)
{
	DWORD	pos;

	switch(origin)
	{
	case SEEK_SET:										// Beginning of file
		pos = 0;
		break;

	case SEEK_CUR:										// Current position of file pointer
		pos = f_tell(stream);
		break;

	case SEEK_END:										// End of file
		pos = f_size(stream);
		break;

	default:
		return -1;
	}
	pos += offset;
    if(f_lseek(stream, pos) != FR_OK)
		return -1;

    if(pos != f_tell(stream))
		return -1;

	return 0;
}


//
// fileno replacement for stdio compatibility
//
int __fileno(FIL *stream)
{
	int		i;

	for(i=0; i<MAX_HANDLES; i++)
	{
		if(&fhTab[i] == stream)
		{
			return i;
		}
	}
	return -1;
}


//
// fread replacement for stdio compatibility
//
size_t	__fread(void *buf, size_t size, size_t count, FIL *stream)
{
    UINT	rc;
	size_t	n = 0;

	while(count--)
	{
        if(f_read(stream, buf, size, &rc) == FR_OK)
        {
        	if(rc != size)
        		break;

        	buf += size;
        	n++;
        }
        else
        	break;
	}
    return n;
}


//
// fwrite replacement for stdio compatibility
//
size_t	__fwrite(const void *buf, size_t size, size_t count, FIL *stream)
{
    UINT	rc;
	size_t	n = 0;

	while(count--)
	{
        if(f_write(stream, buf, size, &rc) == FR_OK)
        {
        	if(rc != size)
        		break;

        	buf += size;
        	n++;
        }
        else
        	break;
	}
    return n;
}



//
// fputc replacement for stdio compatibility
//
int 	__fputc(int c, FIL *stream)
{
	if(f_putc(c, stream) != 1)
		return EOF;

	return c;
}


//
// fgetc replacement for stdio compatibility
//
int 	__fgetc(FIL *stream)
{
	char	c;
	UINT	rc;

	f_read(stream, &c, 1, &rc);
	if (rc != 1)
		return EOF;
	return c;
}



//
// finit :: initialize FATFS and wrapper
//
FATFS	*finit(void)
{
	FRESULT	rc;
	int	i;

	for(i=0; i<MAX_HANDLES; i++)
	{
		fhTab[i].fs = NULL;
	}

	//SD_DeInit();
	f_mount(0, NULL);
	if((rc=f_mount(0, &FileSystemObject)) != FR_OK)
		return NULL;

	return &FileSystemObject;
}



//
// feof :: test for "end of file"
//
int __feof(FIL *stream)
{
	return f_eof(stream);
}

