/**********************************************************************************/
/*    Sample code for Discovery 32F4 board										  */
/*	  Board: STM32F407 Discovery				             					  */
/*    Manufacture: ST		                                                   	  */
/*	  COPYRIGHT (C) 2012														  */
/*    Designed by: Ing. Thomas Winkler			                                  */
/*    Built with CooCox CoIDE v1.5.0  					                          */
/**********************************************************************************/

#include "Discovery.h"
#include "sTimer.h"



#if DEF_EXTI_BUTTON
//
// configure Button B as interrupt Inp (Exti)
//
static void EXTILine0_Config(void)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
  NVIC_InitTypeDef  NVIC_InitStructure;
  EXTI_InitTypeDef	EXTI_InitStructure;

  /* Enable GPIOA clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
  /* Enable SYSCFG clock */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

  /* Configure PA0 pin as input floating */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Pin = BUTTON_B;
  GPIO_Init(BUTTON_PORT, &GPIO_InitStructure);

  /*
  // Connect EXTI Line0 to PA0 pin
  SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource0);

  // Configure EXTI Line0
  EXTI_InitStructure.EXTI_Line = EXTI_Line0;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);


  // Enable and set EXTI Line0 Interrupt to the lowest priority
  NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x01;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x01;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
  */
}
#endif




//
// config GPIO for IEC lines
//
static void InitIEC(void)
{
	NVIC_InitTypeDef	NVIC_InitStructure;
	EXTI_InitTypeDef	EXTI_InitStructure;
	GPIO_InitTypeDef	GPIO_InitStructure;

	/* Enable GPIO Clock */
    RCC_AHB1PeriphClockCmd(IEC_INP_GPIO_CLK, ENABLE);
    RCC_AHB1PeriphClockCmd(IEC_OUT_GPIO_CLK, ENABLE);

	// Set IECo as output.
    GPIO_InitStructure.GPIO_Pin = IEC_ATNo | IEC_CLKo | IEC_DATAo;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_25MHz;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_Init(IEC_OUT, &GPIO_InitStructure);

	// Set IECi as input.
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Pin = IEC_ATNi | IEC_CLKi | IEC_DATAi;
    GPIO_Init(IEC_INP, &GPIO_InitStructure);

    /* Enable SYSCFG clock */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

    // Connect EXTI Line0 to PB0,1,2 pin
    SYSCFG_EXTILineConfig(IEC_INP_EXTI, EXTI_PinSource0);
    SYSCFG_EXTILineConfig(IEC_INP_EXTI, EXTI_PinSource1);
    SYSCFG_EXTILineConfig(IEC_INP_EXTI, EXTI_PinSource2);

    // Configure EXTI Line0,1,2
    EXTI_InitStructure.EXTI_Line = EXTI_Line0 /*| EXTI_Line1 | EXTI_Line2*/;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    /* Enable and set EXTI Line0 Interrupt to the lowest priority */
    //NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn | EXTI1_IRQn | EXTI2_IRQn;
    NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x00;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x00;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
/*  NVIC_InitStructure.NVIC_IRQChannel = EXTI1_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x05;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x05;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
    NVIC_InitStructure.NVIC_IRQChannel = EXTI2_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x05;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x05;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
*/
}




//
// config GPIO for board LEDs
//
static void InitLEDs(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;

	/* Enable GPIO Clock */
    RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

	/* Set LED PIO as output. */
    GPIO_InitStructure.GPIO_Pin = LED_GREEN|LED_ORANGE|LED_RED|LED_BLUE;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_25MHz;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_Init(LED_PORT, &GPIO_InitStructure);
	ResetLEDs();
}




//
// get port and pin for a LED
//
void InitDiscovery()
{
    // Init GPIO
	//
	InitLEDs();
	InitIEC();


    // Button B interrupt
    //
    //EXTILine0_Config();

	InitStimer();
}




//
// Set/Reset a LED on the board
// 0=reset 1-bit, 1=set 1-bit, 2=set bit
//
void SetLED(uint16_t bitValue, int flg)
{
	//bitValue <<= 12;

	switch(flg)
	{
	case 0:
		//LED_PORT->FIOCLR0 = bitValue;
		GPIO_ResetBits(LED_PORT, bitValue);
		break;
	case 1:
		//LED_PORT->FIOSET0 = bitValue;
		GPIO_SetBits(LED_PORT, bitValue);
		break;
	case 3:
		//LED_PORT->FIOSET0 = bitValue;
		GPIO_ToggleBits(LED_PORT, bitValue);
		break;
	default:
		//LED_PORT->FIOPIN0 = bitValue;
		GPIO_Write(LED_PORT, bitValue);
		break;
	}
}



//
// switch off all LED
//
void ResetLEDs(void)
{
	SetLED(ALL_LEDS, 0);
}




//
// delay milliseconds
//
/*
void delay_ms(uint16_t time)
{
	while(time--)
	{
		int i;
		for(i=1; i<1000; ++i)
			;
	}
}
*/


#if F_DIAG
void PrintDiagdata(void)
{
	RCC_ClocksTypeDef RCC_Clocks;
	uint32_t	v;

/*	extern unsigned long _sidata;    // !< Start address for the initialization
	extern unsigned long _sdata;     // !< Start address for the .data section
	extern unsigned long _edata;     // !< End address for the .data section
	extern unsigned long _sbss;      // !< Start address for the .bss section
	extern unsigned long _ebss;      // !< End address for the .bss section

	printf("sidata: %08lx\n", _sidata);
	printf("sdata:  %08lx\n", _sdata);
	printf("edata:  %08lx\n", _edata);
	printf("sbss:   %08lx\n", _sbss);
	printf("ebss:   %08lx\n", _ebss);
*/


	/* SysTick end of count event each 10us */
	printf("\n");
	RCC_GetClocksFreq(&RCC_Clocks);
	v = RCC_Clocks.HCLK_Frequency;
	//printf("HCLK:    %ld\n", v);
	v = RCC_Clocks.PCLK1_Frequency;
	printf("PCLK1:   %ld\n", v);
	v = RCC_Clocks.PCLK2_Frequency;
	printf("PCLK2:   %ld\n", v);
	v = RCC_Clocks.SYSCLK_Frequency;
	printf("SYSCLK:  %ld\n", v);
}
#endif
